<?php
if (! defined('ABSPATH')) exit;

define('WABRIDGE_PLUGIN_DIR', plugin_dir_path(__FILE__));
define('WABRIDGE_VERSION', '1.0.1');


// Register styles and scripts
function wabridge_enqueue_assets()
{
    wp_enqueue_style('wabridge-styles', plugin_dir_url(__FILE__) . '../admin/css/wabridge-styles.css', array(), WABRIDGE_VERSION);
    wp_enqueue_script('wabridge-scripts', plugin_dir_url(__FILE__) . '../admin/js/wabridge-scripts.js', array(), WABRIDGE_VERSION, true);
}
add_action('admin_enqueue_scripts', 'wabridge_enqueue_assets');


add_action('admin_menu', 'WABRIDGE_setup_menu');

function wabridge_setup_menu()
{
    add_menu_page('WhatsApp Notifications', 'WhatsApp Notifications', '', 'send-whatsapp-menu');
    add_submenu_page('send-whatsapp-menu', 'Settings', 'Settings', 'manage_options', 'wabridge-settings', 'wabridge_settings');

    $woocommerce_active = get_option('wabridge_WooCommerce');

    if ('activate' === $woocommerce_active) {
        add_submenu_page('send-whatsapp-menu', 'WhatsApp Custom SMS', 'Custom SMS', 'manage_options', 'wa-custom-sms', 'wabridge_custom_sms');
    }

    add_submenu_page('send-whatsapp-menu', 'History', 'History', 'manage_options', 'wa-msg-history', 'wabridge_msg_history_data');
    add_submenu_page('send-whatsapp-menu', 'SMS Send', 'SMS Send', 'manage_options', 'custom-wa-msg-send', 'wabridge_send_custom_wa_msg');
}

function wabridge_custom_sms()
{
    global $wpdb;
    $table_name = $wpdb->prefix . "options";
    $option_names = array(
        'wabridge_enabled_wc_checkout_form',
        'wabridge_new_order_place',
        'wabridge_pending_order',
        'wabridge_processing_order',
        'wabridge_on_hold_order',
        'wabridge_completed_order',
        'wabridge_cancelled_order',
        'wabridge_failed_order',
        'wabridge_refunded_order',
        'wabridge_checkout_otp'
    );

    $data = get_options(
        $option_names
    );

    $where2 = array(
        'wabridge_failed_order_active_deactive',
        'wabridge_refunded_order_active_deactive',
        'wabridge_cancelled_order_active_deactive',
        'wabridge_completed_order_active_deactive',
        'wabridge_on_hold_order_active_deactive',
        'wabridge_processing_order_active_deactive',
        'wabridge_pending_order_active_deactive',
        'wabridge_new_order_place_active_deactive'
    );

    $data_active = get_options(
        $where2
    );


    $where3 = array('wabridge_failed_order_template_id', 'wabridge_refunded_order_template_id', 'wabridge_cancelled_order_template_id', 'wabridge_completed_order_template_id', 'wabridge_on_hold_order_template_id', 'wabridge_processing_order_template_id', 'wabridge_pending_order_template_id', 'wabridge_new_order_place_template_id');

    $data_template = get_options(
        $where3
    );
    
?>

    <div id="wpbody-content">
        <div class="wrap">
            <h1>WooCommerce Custom Messages</h1>
            <div class="head_css_blk_wa">
                <div class="wcs-nux__notice-logo ">
                    <a href="https://wabridge.com/">

                        <img class="" src="<?php echo esc_url(plugins_url('../admin/images/headerLogo.png', __FILE__)); ?>" width="20%">
                    </a>
                </div>
                <div class="head_links_blk">
                    <a href="https://wabridge.com/" target="_blank">https://wabridge.com</a>
                </div>
            </div>
            <div class="custom_msg_div">
                <h3>Note:</h3>
                <ol class="note_list">
                    <li>if you use WooCommerce plugin then below note read.</li>
                    <li>Use <b>{{user_name}}</b> if you need an user name</li>
                    <li>Use <b>{{Order_id}}</b> if you need an order id</li>
                    <li>Use <b>{{Order_total}}</b> if you need an order total</li>
                    <li>Use <b>Advanced Shipment Tracking for WooCommerce</b> plugin if you need an <b>{{tracking_code}}</b>. <a href="https://wordpress.org/plugins/woo-advanced-shipment-tracking/" target="_black">Click here </a> for download plugin.</li>
                    <li>If you use OTP value then use <b>{{OTP}}</b></li>
                </ol>
            </div>

            <form method="post" action="<?php echo esc_url(sanitize_text_field(wp_unslash(isset($_SERVER['REQUEST_URI']) ? $_SERVER['REQUEST_URI'] : ''))); ?>">


                <table class="form-table">
                    <tbody>
                        <?php
                        if (isset($data) && !empty($data)) {

                            foreach ($data as $option_name => $option_value) {


                                if ($option_name == "wabridge_enabled_wc_checkout_form") {
                                    // Optionally handle this case if needed
                                } else {
                        ?>
                                    <tr>
                                        <th scope="row"><label><?php echo esc_html(ucwords(str_replace("wabridge", "", str_replace("_", " ", $option_name)))); ?></label></th>

                                        <td style="width: 32%">
                                            <textarea name="<?php echo esc_attr($option_name); ?>" type="text" placeholder="Enter <?php echo esc_attr(ucfirst(str_replace("_", " ", $option_name))); ?>" class="regular-text" required><?php echo esc_textarea($option_value); ?></textarea>
                                        </td>

                                        <?php
                                        if (isset($data_template) && !empty($data_template)) {
                                            foreach ($data_template as $template_option_name => $template_option_value) {
                                                if ($option_name . "_template_id" == $template_option_name) {
                                        ?>
                                                    <td style="width: 32%">
                                                        <input name="<?php echo esc_attr($template_option_name); ?>" type="text" placeholder="Enter <?php echo esc_attr(ucfirst(str_replace("_", " ", $template_option_name))); ?>" class="regular-text" required value="<?php echo esc_attr($template_option_value); ?>" />
                                                    </td>
                                        <?php
                                                }
                                            }
                                        }
                                        ?>

                                        <?php

                                        if (isset($data_active) && !empty($data_active)) {
                                            foreach ($data_active as $active_option_name => $active_option_value) {
                                                if ($option_name . "_active_deactive" == $active_option_name) {
                                        ?>
                                                    <td>
                                                        <select name="<?php echo esc_attr($active_option_name); ?>">
                                                            <option value="0" <?php selected($active_option_value, 0); ?>>Deactive</option>
                                                            <option value="1" <?php selected($active_option_value, 1); ?>>Active</option>
                                                        </select>
                                                    </td>
                                        <?php
                                                }
                                            }
                                        }
                                        ?>
                                    </tr>
                        <?php
                                }
                            }
                        }
                        ?>

                    </tbody>
                </table>
                <p class="submit">
                    <input type="submit" name="submit" id="submit" class="button button-primary" value="Update Changes">
                </p>
            </form>



        </div>
        <div class="clear"></div>
    </div>
<?php
}


function WABRIDGE_settings()
{

    $where = array('wabridge_app_key', 'wabridge_auth_key', 'wabridge_device_id', 'wabridge_WooCommerce');

    $data = get_options(
        $where
    );


?>


    <div id="wpbody-content">
        <div class="wrap">
            <h1>Api Settings</h1>

            <div class="head_css_blk_wa">
                <div class="wcs-nux__notice-logo ">
                    <a href="https://wabridge.com/">

                        <img class="" src="<?php echo esc_url(plugins_url('../admin/images/headerLogo.png', __FILE__)); ?>" width="20%">
                    </a>
                </div>
                <div class="head_links_blk">
                    <a href="https://wabridge.com/" target="_blank">https://wabridge.com</a>
                </div>
            </div>
            <div class="notice notice-info">
                <p><strong>Sign up <a href="https://web.wabridge.com/register" target="_blank">Click</a> To Get Api Credentials.</strong></p>
            </div>
            <form method="post" action="<?php echo esc_url(sanitize_text_field(wp_unslash(isset($_SERVER['REQUEST_URI']) ? $_SERVER['REQUEST_URI'] : ''))); ?>">
                <?php wp_nonce_field('send_sms_form_nonce'); ?>
                <table class="form-table">
                    <tbody>
                        <?php
                        if (isset($data) && !empty($data)) {
                            foreach ($data as $setting_key_name => $setting_value) {
                                if ($setting_key_name == "wabridge_msgtype") {
                        ?>
                                    <tr>
                                        <th scope="row"><label><?php esc_html_e('Message Type', 'bulksmsplans-sms-notifications'); ?></label></th>
                                        <td>
                                            <select name="wabridge_msgtype" required="">
                                                <option value=""><?php esc_html_e('Select Type', 'bulksmsplans-sms-notifications'); ?></option>
                                                <option value="1" <?php selected($setting_value, 1); ?>><?php esc_html_e('Text', 'bulksmsplans-sms-notifications'); ?></option>
                                                <option value="2" <?php selected($setting_value, 2); ?>><?php esc_html_e('Unicode', 'bulksmsplans-sms-notifications'); ?></option>
                                            </select>
                                        </td>
                                    </tr>
                                <?php
                                } elseif ($setting_key_name == "wabridge_WooCommerce") {
                                ?>
                                    <tr>
                                        <th scope="row"><label><?php esc_html_e('WooCommerce', 'bulksmsplans-sms-notifications'); ?></label></th>
                                        <td>
                                            <select name="wabridge_WooCommerce" required>
                                                <option value="activate" <?php selected($setting_value, 'activate'); ?>><?php esc_html_e('Activate', 'bulksmsplans-sms-notifications'); ?></option>
                                                <option value="deactivate" <?php selected($setting_value, 'deactivate'); ?>><?php esc_html_e('Deactivate', 'bulksmsplans-sms-notifications'); ?></option>
                                            </select>
                                        </td>
                                    </tr>
                                <?php
                                } else {
                                ?>
                                    <tr>
                                        <th scope="row"><label><?php echo esc_html(ucwords(str_replace('blk', '', str_replace("_", " ", $setting_key_name)))); ?></label></th>
                                        <td>
                                            <input name="<?php echo esc_attr($setting_key_name); ?>" type="text" placeholder="Enter <?php echo esc_attr(ucfirst(str_replace("_", " ", $setting_key_name))); ?>" class="regular-text" required value="<?php echo esc_attr($setting_value); ?>">
                                        </td>
                                    </tr>
                        <?php
                                }
                            }
                        }
                        ?>


                    </tbody>
                </table>

                <p class="submit">
                    <input type="submit" name="submit" id="submit" class="button button-primary" value="Update Changes">

                </p>
            </form>
            <div style="padding: 14px;background: #fff;">
                <h3>Use this shortcode</h3>
                <b>do_shortcode("[WABRIDGE_PLUGIN phone_number='1234567890' msg='Hello!' template_id='xxxxx' variables=[]]");</b>
                <p>This is a one type of sms plugin if you want send message then copy this shortcode and paste on your code and edit phone_number (Type your Number where you want to send message) and edit msg (Write down your message that you want to send a message). Message will be sent where you want to send.</p>
            </div>

        </div>
        <div class="clear"></div>


    </div>
<?php
}


function wabridge_msg_history_data()
{
    global $wpdb;
    $get_table_name = $wpdb->prefix . 'wa_sms_logs'; // Custom table name

    // Sanitize and verify nonce
    $nonce = isset($_GET['_wpnonce']) ? sanitize_key($_GET['_wpnonce']) : '';
    if (!empty($nonce) && !wp_verify_nonce($nonce, 'wabridge_msg_history_nonce')) {
        die('Nonce verification failed');
    }

    // Get current page number
    $pagenum = isset($_GET['pagenum']) ? absint($_GET['pagenum']) : 1;

    // Define limit and offset for pagination
    $limit = 10;
    $offset = ($pagenum - 1) * $limit;

    // Set cache key with page number
    $cache_key = 'wabridge_msg_history_data_' . $pagenum;
    $cached_data = wp_cache_get($cache_key);

    // If no cached data found, fetch fresh data
    if (false === $cached_data) {
        // Get total row count

        // $total = $wpdb->get_var(
        //     "SELECT COUNT(id) FROM " . esc_sql($get_table_name)
        // );

        $total = $wpdb->get_var(
            $wpdb->prepare("SELECT COUNT(id) FROM %s", $get_table_name)
        );


        // Calculate number of pages based on total and limit
        $num_of_pages = ceil($total / $limit);

        // Get the data for the current page
        $questionnaire_data = $wpdb->get_results(
            $wpdb->prepare(
                "SELECT * FROM wp_wa_sms_logs ORDER BY id DESC LIMIT %d, %d",
                $offset,
                $limit
            ),
            OBJECT
        );


        // Get number of rows retrieved
        $rowcount = $wpdb->num_rows;

        // Store data in cache for 1 hour
        $cached_data = array(
            'total' => $total,
            'num_of_pages' => $num_of_pages,
            'questionnaire_data' => $questionnaire_data,
            'rowcount' => $rowcount
        );

        wp_cache_set($cache_key, $cached_data, '', 3600); // Cache for 1 hour
    } else {
        // If cached data exists, use it
        $total = $cached_data['total'];
        $num_of_pages = $cached_data['num_of_pages'];
        $questionnaire_data = $cached_data['questionnaire_data'];
        $rowcount = $cached_data['rowcount'];
    }
?>
    <div class="wrap">
        <h1><?php esc_html_e('SMS Logs List', 'bulksmsplans-sms-notifications'); ?></h1>
        <div class="head_css_blk_wa">
            <div class="wcs-nux__notice-logo">
                <a href="https://wabridge.com/">
                    <img src="<?php echo esc_url(plugins_url('../admin/images/headerLogo.png', __FILE__)); ?>" alt="WA Bridge Logo" width="20%">
                </a>
            </div>
            <div class="head_links_blk">
                <a href="https://wabridge.com/" target="_blank">https://wabridge.com</a>
            </div>
        </div>
        <br>
        <table class="wp-list-table widefat fixed striped">
            <thead>
                <tr>
                    <th scope="col" style="width: 5%;"><?php esc_html_e('S.No', 'bulksmsplans-sms-notifications'); ?></th>
                    <th scope="col"><?php esc_html_e('Request', 'bulksmsplans-sms-notifications'); ?></th>
                    <th scope="col"><?php esc_html_e('Response', 'bulksmsplans-sms-notifications'); ?></th>
                </tr>
            </thead>
            <tbody>
                <?php
                if ($rowcount) {
                    $i = 1;
                    foreach ($questionnaire_data as $singledata) {
                        $json_decode_data = json_decode($singledata->response, true);
                ?>
                        <tr>
                            <td style="text-align: center; padding: 14px;"><?php echo esc_html($i++); ?></td>
                            <td style="text-align: center; padding: 14px;"><?php echo esc_html($singledata->request); ?></td>
                            <td style="text-align: center; padding: 14px;"><?php echo esc_html($singledata->response); ?></td>
                            <td style="text-align: center; padding: 14px; display: none;">
                                <?php
                                $status = $singledata->status;
                                if ($status == "Pending") {
                                    echo "<span class='msg_success' style='background:#F39C12'>" . esc_html($status) . "</span>";
                                } elseif (in_array($status, ["Delivered", "Accepted", "Acknowledged"])) {
                                    echo "<span class='msg_success'>" . esc_html($status) . "</span>";
                                } elseif (in_array($status, ["Rejected", "Failed", "Undeliverable", "Expired", "Unknown"])) {
                                    echo "<span class='msg_error'>" . esc_html($status) . "</span>";
                                } elseif ($status == "DND") {
                                    echo "<span class='msg_error' style='background:#3C8DBC'>" . esc_html($status) . "</span>";
                                }
                                ?>
                            </td>
                        </tr>
                <?php
                    }
                } else {
                    echo "<td colspan='3' align='center'>" . esc_html__('No details(s) found', "bulksmsplans-sms-notifications") . "</td>";
                }
                ?>

            </tbody>
            <tfoot>

                <?php
                $page_links = paginate_links(array(
                    'base' => add_query_arg(array(
                        'pagenum' => '%#%',
                        '_wpnonce' => wp_create_nonce('wabridge_msg_history_nonce') // Adding nonce
                    )),
                    'format' => '',
                    'prev_text' => __('&laquo;', 'bulksmsplans-sms-notifications'),
                    'next_text' => __('&raquo;', 'bulksmsplans-sms-notifications'),
                    'total' => $num_of_pages,
                    'current' => $pagenum
                ));
                if ($page_links) {
                ?>

                    <tr>

                        <td colspan="4">
                            <div class="tablenav tablenav-pages" style="width: 99%;">
                                <div class="tablenav-pages" style="margin: 1em 0"><?php echo wp_kses_post($page_links); ?></div>
                            </div>
                        </td>
                    </tr>
                <?php
                }
                ?>

            </tfoot>
        </table>


    </div>
    </div>
    <div class="clear"></div>


    </div>
<?php
}


function WABRIDGE_send_custom_wa_msg()
{

?>

    <div id="wpbody-content">
        <div class="wrap">
            <h1>Sms Send</h1>
            <div class="head_css_blk_wa">
                <div class="wcs-nux__notice-logo ">
                    <a href="https://wabridge.com/">
                        <img class="" src="<?php echo esc_url(plugins_url('../admin/images/headerLogo.png', __FILE__)); ?>" width="20%">

                    </a>
                </div>
                <div class="head_links_blk">
                    <a href="https://wabridge.com/" target="_blank">https://wabridge.com</a>
                </div>
            </div>
            <form method="post" action="<?php echo esc_url(sanitize_text_field(wp_unslash(isset($_SERVER['REQUEST_URI']) ? $_SERVER['REQUEST_URI'] : ''))); ?>" id="send_wa_msg_form">

                <table class="form-table">
                    <tbody>
                        <input type="hidden" value="send_wa_msg_form" name="action">
                        <tr>
                            <th scope="row"><label>Mobile Number </th>
                            <td><input name="number" type="text" placeholder="Enter Number" class="regular-text " id='txtPhone' required> <span id="spnPhoneStatus"></span></td>
                        </tr>
                        <tr>
                            <th scope="row"><label>Template Id </th>
                            <td><input name="template_id" type="text" placeholder="Enter Template Id" class="regular-text " id='templateId' required> <span id="spnTemplateId"></span></td>
                        </tr>
                        <tr>
                            <th scope="row"><label>Message </th>
                            <td>
                                <textarea name="message" type="text" placeholder="Enter Message" class="regular-text" id="message" required rows="5"></textarea>

                            </td>

                        </tr>


                    </tbody>
                </table>
                <p class="submit">
                    <input type="submit" name="submit" id="submit" class="button button-primary" value="Send">
                </p>
            </form>

        </div>
        <div class="clear"></div>


    </div>

<?php
}

function WABRIDGE_send_msg($phone, $msg, $template_id, $variable_arr = '')
{
    $status = do_shortcode("[WABRIDGE_PLUGIN  phone_number='$phone' msg='$msg' template_id='$template_id' variables='$variable_arr']");

    return $status;
}


// shot code 
add_shortcode('WABRIDGE_PLUGIN', 'WABRIDGE_send_wa_msg_shortcode');


function WABRIDGE_send_wa_msg_shortcode($user_data)
{

    global $wpdb;
    $table_name = $wpdb->prefix . "options";

    $where = array('wabridge_url_key', 'wabridge_app_key', 'wabridge_auth_key', 'wabridge_device_id');

    $data = get_options(
        $where
    );


    if (isset($data) && !empty($data)) {


        $msg_data = urlencode($user_data['msg']);

        $url_custom = "";
        $url = "";
        foreach ($data as $data_key => $get_data) {


            if ($data_key == "wabridge_url_key") {
                $url = base64_decode($get_data) . "?destination_number=" . $user_data['phone_number'] . "&message=" . $msg_data . "&template_id=" . $user_data['template_id'] . "&variables=" . $user_data['variables'];
            } else {

                $url_custom .= "&" . trim(str_replace("wabridge_", '', $data_key)) . "=" . $get_data;

                if (in_array($data_key, ['wabridge_app_key', 'wabridge_auth_key'])) {
                    $url_custom = str_replace("_", '-', $url_custom);
                }
            }
        }
        $full_path_url = $url . $url_custom;

        $response = wp_remote_retrieve_body(wp_remote_get($full_path_url, array('sslverify' => false, 'timeout' => 60)));

        global $wpdb;
        $table_name = $wpdb->prefix . "wa_sms_logs";

        $wpdb->insert(
            $table_name,
            array(
                'request' => wp_json_encode($user_data),
                'response' => $response,
            )
        );

        $response = json_decode($response);
        if ($response->status == true) {
            return 1;
        } else {
            return $response->message;
        }
    }
}
